import { Component, AfterViewInit, OnDestroy, ElementRef, ViewChild, Input, Output, EventEmitter } from '@angular/core';
import { Observable, Subject } from 'rxjs/Rx';


@Component({
  selector: 'process-noding',
  template: "\n    <div class=\"cavas-paper\" [ngStyle]=\"{'background-color': canvasConfig?.backgroundColor, 'width': canvasConfig?.width + 'px', 'height': canvasConfig?.height + 'px'}\">\n      <canvas #myCavas width=\"{{canvasConfig?.width}}\" height=\"{{canvasConfig?.height}}\" [ngClass]=\"{'cursoe-on': cursorFlag, 'cursoe-off': !cursorFlag}\" (mouseleave)=\"mouseLeave($event)\" (mousemove)=\"mouseMove($event)\" (mousedown)=\"mouseDown($event)\"></canvas>\n    </div>\n  ",
  styles: ["\n    .cavas-paper {\n      margin-left: auto;\n      margin-right: auto;\n      border: 1px solid black;\n      -webkit-box-shadow: 2px 4px 2px 0px rgba(0,0,0,0.55);\n      -moz-box-shadow: 2px 4px 2px 0px rgba(0,0,0,0.55);\n      box-shadow: 2px 4px 2px 0px rgba(0,0,0,0.55);\n    }\n\n    .cursoe-on {\n      cursor: pointer;\n    }\n\n    .cursoe-off {\n      cursor: default;\n    }\n  "]
})
export class ProcessNodingComponent implements AfterViewInit, OnDestroy {
  @ViewChild('myCavas') canvasElement:ElementRef;
  @Input() canvasConfig: any;
  @Input() nodes: any;
  @Output() errorNodeNotify: EventEmitter<number> = new EventEmitter<number>();

  drawingTimerHandler;
  blinkingTimerHandler;
  canvasSect;
  cursorFlag = false;
  blinkingFlag;
  context;
  startX;
  startY;
  indexNode;
  errorSpot;
  constructor() {
  }

  ngOnDestroy() {
    if (this.drawingTimerHandler) {
      this.drawingTimerHandler.unsubscribe();
    }

    if (this.blinkingTimerHandler) {
      this.blinkingTimerHandler.unsubscribe();
    }
  }

  ngAfterViewInit() {
    let canvas = this.canvasElement.nativeElement;
    this.context = canvas.getContext("2d");

    this.canvasSect = canvas.getBoundingClientRect();

    this.startY = this.canvasConfig.height / 2;
    this.startX = this.canvasConfig.startX;

    this.drawNormalCircle(-1, this.canvasConfig.terminalCircleConfig.name.start, this.canvasConfig.terminalCircleConfig);

    this.nodes.forEach((node)=> {
      this.drawNormalLine();
      this.drawNormalCircle(node.index, node.name, this.canvasConfig.circleConfig);
    });
    this.drawNormalLine();
    this.drawNormalCircle(-1, this.canvasConfig.terminalCircleConfig.name.end, this.canvasConfig.terminalCircleConfig);

    this.startX = this.canvasConfig.startX;

    this.indexNode = -1;
  }

  draw() {
    let self = this;

    if (this.indexNode === -1) {
      this.drawCircle(this.canvasConfig.terminalCircleConfig, false, () => {
        self.indexNode++;
        self.draw();
      }, 'start');
    } else if (this.indexNode < this.nodes.length) {
      this.drawLine(() => {
        self.drawCircle(this.canvasConfig.circleConfig, this.nodes[self.indexNode].error, () => {
          if (!this.nodes[self.indexNode].error) {
            self.indexNode++;
            self.draw();
          }
        });
      });
    } else {
      this.drawLine(() => {
        self.drawCircle(this.canvasConfig.terminalCircleConfig, false, () => {
        }, 'end');
      });
    }
  }

  drawNormalCircle(index, text, config) {
    this.context.beginPath();
    this.context.lineWidth = config.lineWidth;
    this.context.strokeStyle = this.canvasConfig.lineNormalColor;
    this.context.arc(this.startX + config.radius, this.startY, config.radius, 0 * Math.PI, 2 * Math.PI);
    this.context.stroke();

    this.context.fillStyle = 'white';
    this.context.textAlign = 'center';

    if (index > -1) {
      this.context.font = config.indexFont;
      this.context.fillText(index.toString(), this.startX + config.radius, this.startY + 30);
    }
    this.context.font = config.textFont;
    this.context.fillText(text, this.startX + config.radius, this.startY - config.offset);

    this.startX += 2 * config.radius;
  }

  drawNormalLine() {
    this.context.beginPath();
    this.context.moveTo(this.startX, this.startY);
    this.context.lineTo(this.startX + this.canvasConfig.lineConfig.length, this.startY);
    this.context.lineWidth = this.canvasConfig.lineConfig.lineWidth;
    this.context.strokeStyle = this.canvasConfig.lineNormalColor;
    this.context.stroke();
    this.startX += this.canvasConfig.lineConfig.length;
  }

  drawLine(callback) {
    let lineConfig = JSON.parse(JSON.stringify(this.canvasConfig.lineConfig));
    let steps = lineConfig.length;
    let self = this;
    let fromX = this.startX;

    let timer = Observable.timer(0, lineConfig.speed);
    this.drawingTimerHandler = timer.subscribe(() => {
      //console.log('STEP ' + steps);
      if (steps > 0) {
        self.drawHorizonalLineAnimate(lineConfig.lineWidth, lineConfig.lineColor, fromX);
        steps--;
        fromX += 1;
      } else {
        self.startX = fromX;
        self.drawingTimerHandler.unsubscribe();
        if (callback) {
          callback();
        }
      }
    });
  }

  drawHorizonalLineAnimate(lineWidth, lineColor, fromX) {
    let self = this;
    this.context.beginPath();
    this.context.moveTo(fromX, this.startY);
    this.context.lineTo(fromX + 1, this.startY);
    this.context.lineWidth = lineWidth;
    this.context.strokeStyle = this.canvasConfig.lineColor;
    this.context.stroke();
  }

  drawCircle(config, error, callback, terminalType?) {
    let steps = 20;
    let self = this;
    let fromAng = 1;

    let timer = Observable.timer(0, config.speed);
    this.drawingTimerHandler = timer.subscribe(() => {
      //console.log('fromAng ' + fromAng);
      if (steps > 0) {
        self.drawCircleAnimate(config, error, fromAng);
        steps--;
        fromAng += 0.1;
      } else {

        if (error) {
          self.errorSpot = {
            nodeIndex: this.indexNode,
            x: this.startX + config.radius,
            y: this.startY,
            radius: config.radius
          };

          self.blinkingErrorNode(config);
        } else {
          if (terminalType === undefined) {
            self.fillInColorAndDisplayText(
              this.canvasConfig.lineColor,
              config.radius,
              config.successColor,
              config.indexFont,
              this.nodes[this.indexNode].index.toString(),
              config.textFont,
              this.nodes[this.indexNode].name,
              config.offset
            );
          } else {
            self.fillInColorAndDisplayText(
              this.canvasConfig.lineColor,
              config.radius,
              config.color,
              config.indexFont,
              '',
              '',
              config.name[terminalType],
              config.offset
            );
          }
          self.startX += 2 * config.radius;
        }

        self.drawingTimerHandler.unsubscribe();
        if (callback) {
          callback();
        }
      }
    });
  }

  fillInColorAndDisplayText(fillColor, radius, textColor, indexFont, indexStr, textFont, text, offset) {
    this.context.fillStyle = fillColor;
    this.context.beginPath();
    this.context.arc(this.startX + radius, this.startY, radius, 0, 2 * Math.PI);
    this.context.closePath();
    this.context.fill();

    this.context.fillStyle = textColor;
    this.context.textAlign = 'center';

    this.context.font = indexFont;
    this.context.fillText(indexStr, this.startX + radius, this.startY + 30);
    this.context.font = textFont;
    this.context.fillText(text, this.startX + radius, this.startY - offset);
  }

  blinkingErrorNode(config) {
    let self = this;

    let timer = Observable.timer(0, this.canvasConfig.blinkingSpeed);
    this.blinkingTimerHandler = timer.subscribe(() => {
      this.blinkingFlag = !this.blinkingFlag;

      if (this.blinkingFlag) {
        self.fillInColorAndDisplayText(
          this.canvasConfig.lineErrorColor,
          config.radius,
          config.failedColor,
          config.indexFont,
          this.nodes[this.indexNode].index.toString(),
          config.textFont,
          this.nodes[this.indexNode].name,
          config.offset
        );
      } else {
        self.fillInColorAndDisplayText(
          config.failedColor,
          config.radius,
          this.canvasConfig.lineErrorColor,
          config.indexFont,
          this.nodes[this.indexNode].index.toString(),
          config.textFont,
          this.nodes[this.indexNode].name,
          config.offset
        );
      }
    });
  }



  drawCircleAnimate(config, error, fromAng) {
    this.context.beginPath();
    this.context.lineWidth = config.lineWidth;
    this.context.strokeStyle = error ? this.canvasConfig.lineErrorColor : this.canvasConfig.lineColor;
    this.context.arc(this.startX + config.radius, this.startY, config.radius, fromAng * Math.PI, (fromAng + 0.1) * Math.PI);
    this.context.stroke();
  }

  mouseLeave(e) {
    this.cursorFlag = false;
  }

  mouseDown(e) {
    if (this.cursorFlag) {
      this.showSolution(this.nodes[this.errorSpot.nodeIndex].index);
    }
  }

  mouseMove(e) {
    if (this.errorSpot) {
      let x = e.clientX - this.canvasSect.left, y = e.clientY - this.canvasSect.top;
      this.cursorFlag = Math.sqrt((this.errorSpot.x - x) * (this.errorSpot.x - x) + (this.errorSpot.y - y) * (this.errorSpot.y - y)) < this.errorSpot.radius;
    }
  }

  showSolution(nodeIndex) {
    this.errorNodeNotify.emit(nodeIndex);
  }

}
